@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
 """Downloading utilities for urls"""
+import copy
 import hashlib
 import logging
 from collections import OrderedDict
 from os import remove
 from os.path import splitext, join, exists
 from posixpath import basename
-from typing import Optional, Dict, Iterator, Union, List, Any, Tuple
+from typing import Optional, Dict, Iterator, Union, List, Any, Tuple, Callable
 
 import requests
 import tabulator
 from ratelimit import sleep_and_retry, RateLimitDecorator
 from requests import Request
 from six.moves.urllib.parse import urlsplit, urlunsplit, parse_qsl, urlencode
-from tabulator.exceptions import TabulatorException
 
 from hdx.utilities import raisefrom
 from hdx.utilities.path import get_temp_dir
@@ -182,20 +182,21 @@ class Download(object):
         return full_url, getparams
 
     @staticmethod
-    def hxl_row(headers, hxltags, as_dict=False):
+    def hxl_row(headers, hxltags, dict_form=False):
         # type: (List[str], Dict[str,str], bool) -> Union[List[str], Dict[str,str]]
         """Return HXL tag row for header row given list of headers and dictionary with header to HXL hashtag mappings.
-        Return list or dictionary depending upon teh as_dict argument.
+        Return list or dictionary depending upon the dict_form argument.
 
         Args:
             headers (List[str]): Headers for which to get HXL hashtags
             hxltags (Dict[str,str]): Header to HXL hashtag mapping
+            dict_form (bool): Return dict or list. Defaults to False (list)
 
         Returns:
             Union[List[str],Dict[str,str]]: Return either a list or dictionary conating HXL hashtags
 
         """
-        if as_dict:
+        if dict_form:
             return {header: hxltags.get(header, '') for header in headers}
         else:
             return [hxltags.get(header, '') for header in headers]
@@ -351,7 +352,7 @@ class Download(object):
             self.response = tabulator.Stream(url, **kwargs)
             self.response.open()
             return self.response
-        except TabulatorException as e:
+        except Exception as e:
             raisefrom(DownloadError, 'Getting tabular stream for %s failed!' % url, e)
 
     def get_tabular_rows_as_list(self, url, **kwargs):
@@ -371,20 +372,27 @@ class Download(object):
         """
         return self.get_tabular_stream(url, **kwargs).iter(keyed=False)
 
-    def get_tabular_rows(self, url, headers=1, dict_rows=False, insertions=None, **kwargs):
-        # type: (str, Union[int, List[int], List[str]], bool, Optional[Dict], Any) -> Tuple[List[str],Iterator[Union[List,Dict]]]
+    def get_tabular_rows(self, url, headers=1, dict_form=False, ignore_blank_rows=True, header_insertions=None, row_function=None, **kwargs):
+        # type: (str, Union[int, List[int], List[str]], bool, bool, Optional[List[Tuple[int,str]]], Optional[Callable[[List[str],Union[List,Dict]],Union[List,Dict]]], Any) -> Tuple[List[str],Iterator[Union[List,Dict]]]
         """Returns header of tabular file pointed to by url and an iterator where each row is returned as a list
-        or dictionary depending on the dict_rows argument. Optionally, headers and values can be inserted at specific
-        positions. This is achieved using the insertions argument. If supplied, it must be a dictionary containing
-        the keys "headers" and "functions". "headers" contains a list of tuples of the form (position, header) to be
-        inserted and "functions" is a list of functions each of which takes a parameter extended_rows which contains
-        the row's number, file headers list and the row values list.
+        or dictionary depending on the dict_rows argument. The headers argument is either a row number or list of row
+        numbers (in case of multi-line headers) to be considered as headers (rows start counting at 1), or the actual
+        headers defined as a list of strings. It defaults to 1 and cannot be None.  The dict_form arguments specifies
+        if each row should be returned as a dictionary or a list, defaulting to a list.
+
+        Optionally, headers can be inserted at specific positions. This is achieved using the header_insertions
+        argument. If supplied, it is a list of tuples of the form (position, header) to be inserted. A function is
+        called for each row. If supplied, it takes as arguments: headers (prior to any insertions) and
+        row (which will be in dict or list form depending upon the dict_rows argument) and outputs a modified row
+        or None to ignore the row.
 
         Args:
-            url (str): URL to download
+            url (str): URL or path to read from
             headers (Union[int, List[int], List[str]]): Number of row(s) containing headers or list of headers. Defaults to 1.
-            dict_rows (bool): Return dict (requires headers parameter) or list for each row. Defaults to False (list)
-            insertions (Optional[Dict]): Dictionary with additional headers and functions (see description).
+            dict_form (bool): Return dict or list for each row. Defaults to False (list)
+            ignore_blank_rows (bool): Whether to ignore blank rows. Defaults to True.
+            header_insertions (Optional[List[Tuple[int,str]]]): List of (position, header) to insert. Defaults to None.
+            row_function (Optional[Callable[[List[str],Union[List,Dict]],Union[List,Dict]]]): Function to call for each row. Defaults to None.
             **kwargs:
             file_type (Optional[str]): Type of file. Defaults to inferring.
             delimiter (Optional[str]): Delimiter used for values in each row. Defaults to inferring.
@@ -393,14 +401,34 @@ class Download(object):
             Tuple[List[str],Iterator[Union[List,Dict]]]: Tuple (headers, iterator where each row is a list or dictionary)
 
         """
-        if insertions is not None:
-            kwargs['post_parse'] = insertions['functions']
+        if headers is None:
+            raise DownloadError('Argument headers cannot be None!')
+        if ignore_blank_rows:
+            skip_rows = kwargs.get('skip_rows', list())
+            if {'type': 'preset', 'value': 'blank'} not in skip_rows:
+                skip_rows.append({'type': 'preset', 'value': 'blank'})
+            kwargs['skip_rows'] = skip_rows
         stream = self.get_tabular_stream(url, headers=headers, **kwargs)
-        headers = stream.headers
-        if insertions is not None:
-            for position, header in insertions['headers']:
+        origheaders = stream.headers
+        if header_insertions is None or origheaders is None:
+            headers = origheaders
+        else:
+            headers = copy.deepcopy(origheaders)
+            for position, header in header_insertions:
                 headers.insert(position, header)
-        return headers, stream.iter(keyed=dict_rows)
+        if row_function is None and ignore_blank_rows is False:
+            return headers, stream.iter(keyed=dict_form)
+
+        def get_next():
+            for row in stream.iter(keyed=dict_form):
+                if row_function:
+                    processed_row = row_function(origheaders, row)
+                    if processed_row is not None:
+                        yield processed_row
+                else:
+                    yield row
+
+        return headers, get_next()
 
     def download_tabular_key_value(self, url, **kwargs):
         # type: (str, Any) -> Dict
